package com.android.cideasybridge;
import java.util.HashMap;
import java.util.Iterator;

import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.util.Log;
import android.widget.TextView;
import android.widget.Toast;

/**
 * This class is used for talking to Cid Easy Devices
 * @author Kybele
 * jan 10 2022
 */


public class CidEasyBridge {
    private static final String ACTION_USB_PERMISSION = "com.cideasy.USB_PERMISSION";
    private static final long DEVICE_VENDOR_ID = 4660;
    private static final long DEVICE_PRODUCT_ID = 1;

    private HashMap<UsbDevice, UsbDataBinder> mHashMap = new HashMap<UsbDevice, UsbDataBinder>();
    private UsbManager mUsbManager;
    private PendingIntent mPermissionIntent;
    UsbDevice theDevice = null;

    private Context ctx;

    private TextView textView;

    public CidEasyBridge(Context _ctx) {
        ctx = _ctx;
        mUsbManager = (UsbManager) ctx.getSystemService(Context.USB_SERVICE);
        Connect();

        textView = (TextView) ((MainActivity) ctx).findViewById(R.id.textView);
    }

    protected void Connect() {
        IntentFilter filter;

        filter = new IntentFilter(UsbManager.ACTION_USB_DEVICE_ATTACHED);
        ctx.registerReceiver(mUsbAttachReceiver , filter);
        filter = new IntentFilter(UsbManager.ACTION_USB_DEVICE_DETACHED);
        ctx.registerReceiver(mUsbDetachReceiver , filter);

        mPermissionIntent = PendingIntent.getBroadcast(ctx, 0, new Intent(ACTION_USB_PERMISSION), 0);
        filter = new IntentFilter(ACTION_USB_PERMISSION);
        ctx.registerReceiver(mUsbReceiver, filter);

        traceUsbDevices();
    }

    BroadcastReceiver mUsbDetachReceiver = new BroadcastReceiver() {
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();

            if (UsbManager.ACTION_USB_DEVICE_DETACHED.equals(action)) {
                UsbDevice device = (UsbDevice)intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
                if (device != null) {
                    UsbDataBinder binder = mHashMap.get(device);
                    if (binder != null) {
                        theDevice = null;
                        binder.onDestroy();
                        mHashMap.remove(device);

                        Log.d("CidEasyBridge", "DEVICE DETACHED ");
                    }
                }
            }
        }
    };

    BroadcastReceiver mUsbAttachReceiver = new BroadcastReceiver() {
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();

            if (UsbManager.ACTION_USB_DEVICE_ATTACHED.equals(action)) {
                traceUsbDevices();
                Log.d("CidEasyBridge", "DEVICE ATTACHED ") ;
            }
        }
    };

    private void traceUsbDevices() {
        HashMap<String, UsbDevice> deviceList = mUsbManager.getDeviceList();
        Iterator<UsbDevice> deviceIterator = deviceList.values().iterator();
        while(deviceIterator.hasNext()){
            UsbDevice device = deviceIterator.next();
            if ((device.getVendorId() == DEVICE_VENDOR_ID) && ((device.getProductId() == DEVICE_PRODUCT_ID))) {
                Log.d("CidEasyBridge","DEVICE FOUND - PERMISSIN REQUESTED ");
                mUsbManager.requestPermission(device, mPermissionIntent);
            }
        }
    }

    private final BroadcastReceiver mUsbReceiver = new BroadcastReceiver() {
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if (ACTION_USB_PERMISSION.equals(action)) {
                synchronized (this) {
                    UsbDevice device = (UsbDevice) intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
                    if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false)) {
                        if (device != null) {
                            theDevice = device;

                            UsbDataBinder binder = new UsbDataBinder(mUsbManager, device);
                            mHashMap.put(device, binder);
                            Log.d("CidEasyBridge - ", "PERMISSION GRANTED " + theDevice);

                            binder.StartCidEasyThread(ctx);

                            binder.setBinderListener(new UsbDataBinder.binderListener() {
                                @Override
                                public void onBinderLineEvent(char thePort, String theEvent) {
                                    final String sMessage = "LINE EVENT "+theEvent+" ON PORT : "+thePort+"\r\n";
                                    Log.d("CEBridge",sMessage);
                                    ((MainActivity) ctx).runOnUiThread(new Runnable() {
                                        public void run() {
                                            textView.append(sMessage);
                                            //Toast.makeText((MainActivity) ctx,(String)sMessage, Toast.LENGTH_SHORT).show();
                                        }
                                    });
                                }

                                @Override
                                public void onBinderCliReceived(char thePort, String theDateTime, String theCaller, String theCallee) {
                                    final String sMessage = "Caller:"+theCaller+",   Dt&Tm:"+theDateTime+",  Port:"+thePort+",   Callee:"+theCallee+"\r\n";
                                    Log.d("CEBridge",sMessage);
                                    ((MainActivity) ctx).runOnUiThread(new Runnable() {
                                        public void run() {
                                            textView.append(sMessage);
                                            //Toast.makeText((MainActivity) ctx,(String)sMessage, Toast.LENGTH_SHORT).show();
                                        }
                                    });

                                }
                            });
                        }
                    } else {
                        Log.d("CidEasyBridge - ", "PERMISSION DENIED " + device);
                    }
                }
            }
        }
    };

}